'''
    原神随机事件
'''
from random import choice
from typing import Literal
from pydantic import BaseModel
from ayaka import AyakaCat, load_data_from_file
from .data import downloader

cat = AyakaCat("原神随机事件")


class Group(BaseModel):
    type: Literal["Group"] = "Group"
    rules: list[str]
    parts: list["Part"]

    def get_value(self, rule: str = None, data: dict = None):
        params = {k: p.choice() for p in self.parts for k in p.names}

        # 内部参数 + 外部参数
        if data:
            params.update(data)

        # 内部规则 or 外部规则
        if not rule:
            if not self.rules:
                return ""
            rule = choice(self.rules)

        return rule.format(**params)


class Part(BaseModel):
    type: Literal["Part"] = "Part"
    names: list[str]
    values: list[str | Group]

    def choice(self):
        if not self.values:
            return ""
        value = choice(self.values)
        if isinstance(value, Group):
            return value.get_value()
        return value


# 很重要
Group.update_forward_refs()

'''
简单介绍原理：
1.每个句子都可以视为一个Group
2.Group由多个Part、以自定义的rule拼接而成，而每个Part都有自己的名字，通过名字来确定他们在拼接规则中的位置，例如rule = {名字}在{地点}做{动作}
3.rule可以有多个，Group随机选择一个rule进行拼接
4.Part内存有一个数据元素池，Part随机从数据元素池中抽取一个元素作为值
5.Part的数据元素池中的元素，可以是字符串，也可以是Group
6.开始套娃
'''

group = Group(rules=[], parts=[])


@downloader.on_finish
async def load():
    global group
    path = downloader.BASE_DIR / "原神随机事件.json"
    data = load_data_from_file(path)
    group = Group(**data)


@cat.on_cmd(cmds="原神随机事件")
async def _():
    '''带参数可以自定义事件，例如：原神随机事件 喝水'''
    event = str(cat.current.arg)
    if event:
        await cat.send(group.get_value(data={"事件": event}))
    else:
        await cat.send(group.get_value())



'''
    签到模块
'''
import datetime
from ayaka import AyakaCat, AyakaUserDB
from .bag import get_money
from .data import config

cat = AyakaCat('签到')


class LastDate(AyakaUserDB):
    __table_name__ = "checkin"
    value: str = ""


@cat.on_cmd(cmds=['checkin', '签到'], always=True)
async def checkin():
    last_date = LastDate.select_one(
        group_id=cat.current.session_id,
        user_id=cat.current.sender_id
    )

    date = str(datetime.datetime.now().date())

    if date == last_date.value:
        await cat.send(f"[{cat.current.sender_name}] 今天已经签到过了")
        return

    last_date.value = date

    # 签到奖励
    money = get_money(cat.current.session_id, cat.current.sender_id)
    money.value += config.checkin_reward
    await cat.send(f"[{cat.current.sender_name}] 签到成功，系统奖励 {config.checkin_reward}金")
    await cat.send(f"[{cat.current.sender_name}] 当前拥有 {money.value}金")



'''
    接龙，多个词库可选择
'''
import re
from random import choice
from pypinyin import lazy_pinyin
from pydantic import BaseModel, Field
from ayaka import AyakaCat, AyakaDB, load_data_from_file
from .bag import get_money
from .data import downloader, config

cat = AyakaCat("接龙管理")
cat.help = '''接龙，在聊天时静默运行'''


class Dragon(BaseModel):
    '''接龙词库'''
    name: str
    use: bool = True
    words: list[str] = []
    pinyin_dict: dict[str, list] = {}

    def __init__(self, **data) -> None:
        super().__init__(**data)

        if not self.pinyin_dict:
            self.get_dict()

    def get_dict(self):
        for word in self.words:
            # 获取首字的拼音
            p = lazy_pinyin(word)[0]
            if p not in self.pinyin_dict:
                self.pinyin_dict[p] = []
            self.pinyin_dict[p].append(word)

    def check(self, word: str):
        return word in self.words

    def next(self, word: str):
        # 获取末字的拼音
        p = lazy_pinyin(word)[-1]
        words: list[str] = self.pinyin_dict.get(p)
        if not words:
            return ""
        return choice(words)


class DragonUserData(AyakaDB):
    '''用户数据'''
    __table_name__ = "dragon_user_data"
    group_id: str = Field(extra=AyakaDB.__primary_key__)
    user_id: str = Field(extra=AyakaDB.__primary_key__)
    dragon_name: str = Field(extra=AyakaDB.__primary_key__)
    cnt: int = 0

    @classmethod
    def get(cls, dragon_name: str):
        return cls.select_one(
            dragon_name=dragon_name,
            group_id=cat.current.session_id,
            user_id=cat.current.sender_id
        )


dragon_list: list[Dragon] = []


@downloader.on_finish
async def finish():
    path = downloader.BASE_DIR / "接龙"
    for p in path.iterdir():
        try:
            words = load_data_from_file(p)
            dragon_list.append(Dragon(name=p.stem, words=words))
        except:
            pass


zh = re.compile(r"[\u4e00-\u9fff]+")


@cat.on_text(always=True)
async def handle():
    '''自动接龙'''
    text = cat.current.arg
    r = zh.search(text)
    if not r:
        return

    word = r.group()

    for dragon in dragon_list:
        # 跳过不启用的接龙
        if not dragon.use:
            continue

        # 当前词语符合接龙词库
        if dragon.check(word):

            # 上次接龙
            last = cat.cache.get("dragon", {}).get(dragon.name, "")

            # 成功接龙
            if last and word:
                p1 = lazy_pinyin(last)[-1]
                p2 = lazy_pinyin(word)[0]
                if p1 == p2:
                    # 修改金钱
                    usermoney = get_money(
                        group_id=cat.current.session_id, user_id=cat.current.sender_id)
                    usermoney.value += config.dragon_reward
                    await cat.send(f"[{cat.current.sender_name}] 接龙成功！奖励{config.dragon_reward}金")

                    # 修改记录
                    user_data = DragonUserData.get(dragon.name)
                    user_data.cnt += 1

            # 无论是否成功接龙都发送下一个词
            word = dragon.next(word)
            cat.cache.setdefault("dragon", {})
            cat.cache["dragon"][dragon.name] = word
            if not word:
                word = choice(["%$#*-_", "你赢了", "接不上来..."])
            await cat.send(word)
            break


cat.set_wakeup_cmds(cmds="接龙管理")
cat.set_rest_cmds(cmds=["exit", "退出"])


@cat.on_cmd(cmds="list", states="idle")
async def list_all():
    '''列出所有词库'''
    items = ["所有词库："]
    for dragon in dragon_list:
        if dragon.use:
            items.append(f"[{dragon.name}] 正在使用")
        else:
            items.append(f"[{dragon.name}]")
    await cat.send("\n".join(items))


@cat.on_cmd(cmds="data", states="idle")
async def show_data():
    '''展示你的答题数据'''
    gid = cat.current.session_id
    uid = cat.current.sender_id

    user_datas = DragonUserData.select_many(group_id=gid, user_id=uid)

    if user_datas:
        info = "\n".join(
            f"[{u.dragon_name}] 接龙次数 {u.cnt}"
            for u in user_datas
        )
    else:
        info = "你还没有用过我...T_T"

    await cat.send(info)


@cat.on_cmd(cmds="rank", states="idle")
async def show_rank():
    '''展示排行榜'''
    data: dict[str, list[DragonUserData]] = {}

    # SELECT * from dragon_user_data ORDER BY dragon_name, cnt DESC
    user_datas = DragonUserData.select_many(group_id=cat.current.session_id)
    for user_data in user_datas:
        if user_data.dragon_name not in data:
            data[user_data.dragon_name] = []
        data[user_data.dragon_name].append(user_data)

    # 无人使用
    for dragon in dragon_list:
        for user_data in user_datas:
            if user_data.dragon_name == dragon.name:
                break
        else:
            data[dragon.name] = []

    users = await cat.get_users()
    users = {u.id: u.name for u in users}

    info = "排行榜\n"
    for dragon_name, datas in data.items():
        info += f"\n[{dragon_name}]\n"
        if not datas:
            info += f"  - 暂时没人使用过...T_T\n"
        else:
            datas.sort(key=lambda x: x.cnt, reverse=1)
            for d in datas[:5]:
                info += f"  - [{users[d.user_id]}] 接龙次数 {d.cnt}\n"
    await cat.send(info.strip())



from enum import Enum
from random import randint, choice, shuffle
from ayaka import AyakaCat, AyakaSession

cat = AyakaCat("印加宝藏")
cat.help = "欢迎使用印加宝藏2.0"

ruledoc = '''1. 前进，玩家翻开一张卡牌
2. 撤退，玩家沿着来时的路径原路返回
3. 遇到宝石，玩家平分宝石，剩余的宝石留在原地
4. 遇到遗物，当且仅当一名玩家撤退时可从卡片上获得遗物
5. 在前进时遇到怪物，第二次遇到将被驱逐出神殿，丢失此轮在神殿中获得的一切收益
6. 前两个被带出的遗物计5分，后3个被带出的遗物计10分'''


warnings = [
    '人类，离开此地！',
    '吾等奉命守护此地，驱逐一切外来者',
    '别让我第二次看到你'
]

findArtifact = [
    '就是这个！谢谢你帮我找到它！能将这孩子带出神殿么？', '这是……我留下的……遗物？',
    '虽说是我最重要的东西，但埋藏在神殿里的话根本就没意义嘛！',
    '遗物流通于世，这个世界一定会发生某些变化吧~真是期待呢~',
    '遗物拥有特殊的力量，沉睡在神殿里的话是没办法苏醒的呢~'
]

goddess = [
    '有且仅有一个人撤退才能带走遗物哦~趁别人不知道的时候偷偷撤退吧！',
    '说起来，我曾在神殿一共制造了5个魔偶呢~欸？难道说已经见过了？危险了啊，赶快离开比较好哦！'
    '帮我带出遗物的话，宝石什么的，要多少有多少哦~'
    '不能平分的东西为什么不能被带走呢？啊，难道是因为自己得不到的东西也不能让别人得到？'
    '沿原路返回的时候可以拿到来时没能平分的宝石和遗物哦~',
    '<Turquoise>有1积分，<Obsidian>有5积分，<Gold>竟然有10积分吗！？'
    '遗物只要出现了一次就不会再出现了呢~'
    '就算发生的概率再低，对于面对的人来说，概率就是100%呢'
    '只要尺度足够大，一切事件都必然发生，那么，谁是幸运儿呢？'
    '幸运或者不幸，对我来说都没有意义，那是从人类的角度才存在的东西'
]

deads = [
    '下次不要这么贪了哦~'
    '多亏了你们，我重新获得了这只魔偶的控制权呢，你们不会再遇到这孩子了',
    '该说是有勇气呢还是无谋呢还是说盲目自信呢……不是很懂你们这些人类。'
    '不要放弃，下次更加谨慎一点吧！'
]

treasures = {
    'Turquoise': {
        'number': 0,
        'value': 1
    },
    'Obsidian': {
        'number': 0,
        'value': 5
    },
    'Gold': {
        'number': 0,
        'value': 10
    },
    'Artifact': {
        'number': 0,
        'value': 5
    }
}


class Card:
    class Type(Enum):
        TEMPLE = 0,
        JEWEL = 1,
        MONSTER = 2,
        ARTIFACT = 3

        def ToString(self):
            if self is self.TEMPLE:
                return 'Temple'
            elif self is self.JEWEL:
                return 'Jewel'
            elif self is self.MONSTER:
                return 'Monster'
            elif self is self.ARTIFACT:
                return 'Artifact'
            else:
                raise ValueError

    def __init__(self, ctype, name=None, number=1, value=0):
        self.ctype = ctype
        self.name = name if name else ctype.ToString()
        self.number = number
        self.value = value


class Deck:
    def __init__(self, ctype='Quest'):
        self.cardset: list[Card] = []
        if ctype == 'Quest':
            for i in range(5):
                self.cardset.append(
                    Card(Card.Type.JEWEL, 'Gold', number=randint(10, 15), value=10))
                self.cardset.append(
                    Card(Card.Type.JEWEL, 'Obsidian', number=randint(10, 15), value=5))
                self.cardset.append(
                    Card(Card.Type.JEWEL, 'Turquoise', number=randint(10, 15), value=1))
                self.cardset.append(Card(Card.Type.ARTIFACT, value=5))
            for i in range(3):
                self.cardset.append(Card(Card.Type.MONSTER, 'Viper'))
                self.cardset.append(Card(Card.Type.MONSTER, 'Spider'))
                self.cardset.append(Card(Card.Type.MONSTER, 'Mummy'))
                self.cardset.append(Card(Card.Type.MONSTER, 'Flame'))
                self.cardset.append(Card(Card.Type.MONSTER, 'Collapse'))
        elif ctype == 'Temple':
            self.cardset.append(Card(Card.Type.TEMPLE, '第一神殿'))
            self.cardset.append(Card(Card.Type.TEMPLE, '第二神殿'))
            self.cardset.append(Card(Card.Type.TEMPLE, '第三神殿'))
            self.cardset.append(Card(Card.Type.TEMPLE, '第四神殿'))
            self.cardset.append(Card(Card.Type.TEMPLE, '第五神殿'))
        shuffle(self.cardset)

    def Draw(self):
        card = choice(self.cardset)
        self.cardset.remove(card)
        return card

    def Remove(self, name):
        self.cardset = [card for card in self.cardset if card.name != name]

    def DrawArtifact(self):
        card = choice(self.cardset)
        while card.ctype is not Card.Type.ARTIFACT:
            card = choice(self.cardset)
        self.cardset.remove(card)
        return card

    def DrawJewel(self):
        card = choice(self.cardset)
        while card.ctype is Card.Type.MONSTER:
            card = choice(self.cardset)
        self.cardset.remove(card)
        return card


class Incan:
    def __init__(self) -> None:
        self.reset()

    def reset(self):
        self.members = {}
        self.round = 0
        self.route = []
        self.deck = Deck()
        self.monsters = []
        self.artifact = 0
        self.acquiredArtifact = 0
        self.turn = 0
        self.temples = Deck('Temple')


def InitPlayer(self: Incan, uid, name):
    from copy import deepcopy
    self.members[uid] = {
        'status': 0,
        'name': name,
        'treasures': treasures,
        'income': deepcopy(treasures)
    }


async def EnterNextRound(model: Incan):
    model.turn = 0
    model.monsters.clear()
    model.route.clear()
    for i in range(model.artifact):
        model.deck.DrawArtifact()

    for uid in model.members:
        model.members[uid]['status'] = 0
        for name in model.members[uid]['treasures']:
            model.members[uid]['income'][name]['number'] += model.members[uid]['treasures'][name]['number']
            model.members[uid]['treasures'][name]['number'] = 0

    model.round += 1
    if model.round == 5:
        return await Clearing(model)
    else:
        await cat.send(f'第{model.round}轮结束')
        await cat.send(f'第{model.round+1}轮：{model.temples.Draw().name}')


async def Clearing(model: Incan):
    alternate_uid = []
    alternate_value = 0

    for uid, member in model.members.items():
        income = 0
        for name, jewel in member['income'].items():
            income += jewel['number'] * jewel['value']
        if income > alternate_value:
            alternate_uid.clear()
            alternate_uid.append(uid)
            alternate_value = income
        elif income == alternate_value and income > 0:
            alternate_uid.append(uid)

    if alternate_value > 0:
        relic = -1
        winner = []
        for uid in alternate_uid:
            if model.members[uid]['income']['Artifact']['number'] > relic:
                relic = model.members[uid]['income']['Artifact']['number'] > relic
                winner.clear()
                winner.append(uid)
            elif relic == model.members[uid]['income']['Artifact']['number'] and relic > 0:
                winner.append(uid)

        for uid in winner:
            trophy = []
            for name, jewel in model.members[uid]['income'].items():
                trophy.append(f'<{name}>{jewel["number"]}枚')
            await cat.send(f'<{model.members[uid]["name"]}>带着{", ".join(trophy)}获得了胜利')
            await cat.send('游戏结束~')

    else:
        await cat.send('有勇气才能获得宝石哦！')
        await cat.send('游戏结束~')

    return True


async def DoRetreat(model: Incan):
    runaways = [
        uid for uid in model.members if model.members[uid]['status'] == 2]
    num = len(runaways)
    if num == 0:
        return

    for card in model.route:
        for uid in runaways:
            model.members[uid]['treasures'][card.name]['number'] += card.number // num
        if card.ctype is Card.Type.ARTIFACT and num == 1:
            model.acquiredArtifact += 1
            if model.acquiredArtifact > 2:
                model.members[uid]['treasures'][card.name]['value'] = 10
        card.number = card.number % num

    names = ",".join(f'<{model.members[uid]["name"]}>' for uid in runaways)
    await cat.send(f'{names}放弃了冒险')


async def send_info(turn, result, tips_giver, tips):
    await cat.send(f'第{turn}回合, {result}\n<{tips_giver}> {tips}')


async def Gaming(model: Incan):
    # CheckTurnEnd
    for uid in model.members:
        if model.members[uid]['status'] == 0:
            return

    model.turn += 1
    await DoRetreat(model)
    for uid in model.members:
        if model.members[uid]['status'] == 1:
            model.members[uid]['status'] = 0
        elif model.members[uid]['status'] == 2:
            model.members[uid]['status'] = 3

    adventures = [
        uid for uid in model.members if model.members[uid]['status'] == 0]

    if adventures:
        card = None
        if model.turn == 1:
            card = model.deck.DrawJewel()
        else:
            card = model.deck.Draw()
        if card.ctype is Card.Type.MONSTER:
            if card.name in model.monsters:
                for uid in adventures:
                    for jewel in model.members[uid]['treasures'].values():
                        jewel['number'] = 0

                names = ",".join(
                    f'<{model.members[uid]["name"]}>' for uid in adventures)
                await send_info(model.turn, f"{names}被<{card.name}>驱逐出神殿，一无所获", "概率女神", choice(deads))

                model.deck = Deck()
                model.deck.Remove(card.name)
                return await EnterNextRound(model)
            else:
                model.monsters.append(card.name)
                await send_info(model.turn, f"发现了来自<{card.name}>的警告", card.name, choice(warnings))
        elif card.ctype is Card.Type.JEWEL:
            await send_info(model.turn, f"发现了宝石<{card.name}>{card.number}枚", "概率女神", choice(goddess))
            num = len(adventures)
            for uid in adventures:
                model.members[uid]['treasures'][card.name]['number'] += card.number // num
            card.number = card.number % num
            model.route.append(card)
        elif card.ctype is Card.Type.ARTIFACT:
            model.route.append(card)
            model.artifact += 1
            await send_info(model.turn, f"发现了遗物<{card.name}>", "概率女神", choice(findArtifact))
    else:
        model.deck = Deck()
        return await EnterNextRound(model)


@cat.on_cmd(cmds=["incan", "印加", "印加宝藏"])
async def game_entrance():
    '''唤醒猫猫'''
    await cat.wakeup("room")

    # 初始化模型
    model = cat.get_data(Incan)
    model.reset()

    # 操作
    InitPlayer(model, cat.current.sender_id, cat.current.sender_name)
    await cat.send(ruledoc)


@cat.on_cmd(cmds=['exit', 'quit', "退出"], states="*")
async def exit_incan():
    '''退出游戏'''
    cat.remove_listener()
    await cat.send('游戏结束~下次再见~')
    await cat.rest()


@cat.on_cmd(cmds=['go', 'forward'], states="gaming")
async def go_handle():
    '''前进'''
    origin = cat.current.event.origin
    if origin:
        await cat.base_send(origin.session, "收到")

    model = cat.get_data(Incan)
    uid = cat.current.sender_id
    if model.members[uid]['status'] == 0:
        model.members[uid]['status'] = 1
    if await Gaming(model):
        await exit_incan()


@cat.on_cmd(cmds=['back', 'retreat', 'escape'], states="gaming")
async def handle():
    '''撤退'''
    origin = cat.current.event.origin
    if origin:
        await cat.base_send(origin.session, "收到")

    model = cat.get_data(Incan)
    uid = cat.current.sender_id
    if model.members[uid]['status'] == 0:
        model.members[uid]['status'] = 2
    if await Gaming(model):
        await exit_incan()


@cat.on_cmd(cmds=['start', 'run'], states="room")
async def handle():
    '''开始游戏'''
    model = cat.get_data(Incan)
    cat.state = "gaming"

    await cat.send('游戏开始，输入[go/back]决定前进/撤退，此指令支持私聊我发出哦~')
    await cat.send(f'第1轮：{model.temples.Draw().name}')

    for uid in model.members:
        cat.add_listener(AyakaSession(type="private", id=uid))


@cat.on_cmd(cmds="join", states="room")
async def handle():
    '''加入游戏'''
    model = cat.get_data(Incan)
    name = cat.current.sender_name
    uid = cat.current.sender_id

    if uid in model.members:
        await cat.send(f'{name}已经在小队中了，无需重复加入')
    else:
        InitPlayer(model, uid, name)
        await cat.send(f'<{name}>加入了小队，当前小队共{len(model.members)}人。')


@cat.on_cmd(cmds=["status", "状态"], states="room")
async def handle():
    '''查看状态'''
    model = cat.get_data(Incan)
    ans = f'队伍玩家有：<{">, <".join([model.members[uid]["name"] for uid in model.members])}>'
    await cat.send(ans)


@cat.on_cmd(cmds=["status", "状态"], states="gaming")
async def handle():
    '''查看状态'''
    model = cat.get_data(Incan)

    def get_state_s(uid):
        if model.members[uid]['status'] == 0:
            return '还在迷茫中'
        if model.members[uid]['status'] == 3:
            return '放弃冒险了'
        return '决定好了'

    status = '角色状态：'
    for uid in model.members:
        state = get_state_s(uid)
        status += f'<{model.members[uid]["name"]}> {state}\n'

    if model.monsters:
        status += f'警告：\n<{">, <".join(model.monsters)}>'
    else:
        status += f'目前没有收到任何警告'

    await cat.send(status)


@cat.on_cmd(cmds=['rule', 'document', 'doc'], states="*")
async def handle():
    '''查看规则'''
    await cat.send(ruledoc)



'''
    谁是卧底？
'''
from random import choice, randint
from ayaka import AyakaCat, load_data_from_file, AyakaSession
from .data import downloader


words_list: list[tuple[str, str]] = []


@downloader.on_finish
async def get_words():
    path = downloader.BASE_DIR / "谁是卧底.txt"
    items = load_data_from_file(path)
    words_list.extend(item.split(" ") for item in items)


cat = AyakaCat("谁是卧底")
cat.help = '''
至少4人游玩，游玩前请加bot好友，否则无法通过私聊告知关键词
参与玩家的群名片不要重名，否则会产生非预期的错误=_=||
卧底只有一个
'''


class Player:
    def __init__(self, uid: int, name: str) -> None:
        self.uid = uid
        self.name = name
        self.word = ""
        self.num = 0

        self.vote_cnt = 0
        self.vote_to: Player = None
        self.is_suspect = False
        self.out = False

    def __str__(self):
        return f"[{self.name}]"

    def clear(self):
        self.vote_cnt = 0
        self.vote_to = None
        self.is_suspect = False
        self.out = False

    def set_normal(self, word: str):
        self.clear()
        self.word = word

    def set_suspect(self, word: str):
        self.clear()
        self.word = word
        self.is_suspect = True

    def clear_vote(self):
        self.vote_to = None
        self.vote_cnt = 0

    @property
    def state(self):
        if self.out:
            return f"[{self.name}] 出局"

        if self.vote_to:
            return f"[{self.name}] 已投"

        return f"[{self.name}] 未投"


class Game:
    def __init__(self) -> None:
        self.players: list[Player] = []

    @property
    def player_cnt(self):
        return len(self.players)

    @property
    def no_out_players(self):
        return [p for p in self.players if not p.out]

    @property
    def voted_end(self):
        if all(p.vote_to for p in self.no_out_players):
            return True

        # 或者有人已获得半数以上的票数
        cnt = int(len(self.no_out_players)/2)
        for p in self.no_out_players:
            if p.vote_cnt > cnt:
                return True

    @property
    def players_state(self):
        items = ["投票情况："]
        items.extend(p.state for p in self.players)
        return "\n".join(items)

    @property
    def vote_info(self):
        items = ["得票情况："]
        items.extend(
            f"[{p.num}] [{p.name}] {p.vote_cnt}" for p in self.no_out_players)
        return "\n".join(items)

    @property
    def room_info(self):
        items = ["房间成员："]
        items.extend(str(p) for p in self.players)
        return "\n".join(items)

    def get_player(self, uid: int):
        for p in self.players:
            if p.uid == uid:
                return p

    def join(self, uid: int, name: str):
        p = self.get_player(uid)
        if p:
            return False, f"{p} 已在房间中"
        p = Player(uid, name)
        self.players.append(p)
        return True, f"{p} 加入房间"

    def leave(self, uid: int):
        p = self.get_player(uid)
        if not p:
            return False, f"({uid}) 不在房间中"
        self.players.remove(p)
        return True, f"{p} 离开房间"

    def get_words(self):
        normal, fake = choice(words_list)

        # # 有可能翻转
        # if randint(0, 1):
        #     normal, fake = fake, normal

        return normal, fake

    def start(self):
        if self.player_cnt < 4:
            return False, "至少需要4人才能开始游戏"

        normal, fake = self.get_words()

        # 初始化状态
        for i, p in enumerate(self.players):
            p.set_normal(normal)
            p.num = i+1

        # 随机卧底
        i = randint(0, self.player_cnt - 1)
        self.players[i].set_suspect(fake)

        return True, "游戏开始"

    def vote(self, uid: int, vote_to_uid: int):
        # 参数校验
        src = self.get_player(uid)
        if not src:
            return False, f"({uid}) 没有加入游戏"

        obj = self.get_player(vote_to_uid)
        if not obj:
            return False, f"({vote_to_uid}) 没有加入游戏"

        # 不可重投
        if src.vote_to:
            return False, f"{src} 已经投票过了"

        # 已出局的不能投票
        if src.out:
            return False, f"{src} 已经出局了"

        # 不能投给已出局的
        if obj.out:
            return False, f"{obj} 已经出局了"

        src.vote_to = obj
        obj.vote_cnt += 1
        return True, f"{src} 投票给了 {obj}"

    def kickout(self):
        info = self.vote_info + "\n\n"
        ps = self.no_out_players

        # 取最高者
        ps.sort(key=lambda p: p.vote_cnt, reverse=True)
        vote_cnt = ps[0].vote_cnt
        for i, p in enumerate(ps):
            if p.vote_cnt < vote_cnt:
                ps = ps[:i]
                break

        # 清理本轮投票
        for p in self.no_out_players:
            p.clear_vote()

        # 只票出一人，正常结算
        if len(ps) == 1:
            p = ps[0]
            p.out = True
            info += f"{p} 出局！"
            return True, info

        # 平票
        info += "平局！"
        return False, info

    def check_end(self):
        for p in self.players:
            if p.is_suspect and p.out:
                return True, "普通人赢了！"
        if len(self.no_out_players) <= 2:
            return True, "卧底赢了！"
        return False, ""

    def conclude(self):
        ps = [p for p in self.players if not p.is_suspect]
        word = ps[0].word

        items = []
        items.append(f"普通人的关键词： {word}")
        items.append(" ".join(str(p) for p in ps) + " 是普通人！")

        for p in self.players:
            if p.is_suspect:
                break

        items.append(f"卧底的关键词： {p.word}")
        items.append(f"{p} 是卧底！")
        return "\n".join(items)


async def check_friend(uid: int):
    try:
        session = AyakaSession(type="private", id=uid)
        await cat.base_send(session, "好友测试，无需回复")
    except:
        return False
    else:
        return True


@cat.on_cmd(cmds="谁是卧底")
async def cat_entrance():
    '''唤醒猫猫'''
    await cat.wakeup("room")
    game = cat.get_data(Game)
    await cat.send_help()
    game.players = []
    await join()


@cat.on_cmd(cmds=["exit", "退出"], states="room")
async def exit_room():
    '''关闭游戏'''
    await cat.rest()


@cat.on_cmd(cmds=["exit", "退出"], states="play")
async def exit_play():
    await cat.send("游戏已开始，你确定要终结游戏吗？请使用命令：强制退出")


@cat.on_cmd(cmds=["join", "加入"], states="room")
async def join():
    '''加入房间'''
    game = cat.get_data(Game)
    # 校验好友
    if not await check_friend(cat.current.sender_id):
        await cat.send("只有bot的好友才可以加入房间，因为游戏需要私聊关键词")
        return

    f, info = game.join(cat.current.sender_id, cat.current.sender_name)
    await cat.send(info)


@cat.on_cmd(cmds=["leave", "离开"], states="room")
async def leave():
    '''离开房间'''
    game = cat.get_data(Game)
    f, info = game.leave(cat.current.sender_id)
    await cat.send(info)

    if f and game.player_cnt == 0:
        await cat.rest()


@cat.on_cmd(cmds=["start", "begin", "开始"], states="room")
async def start():
    '''开始游戏'''
    game = cat.get_data(Game)
    f, info = game.start()
    await cat.send(info)

    # 启动失败
    if not f:
        return

    cat.state = "play"
    for p in game.players:
        session = AyakaSession(type="private", id=p.uid)
        await cat.base_send(session, p.word)


@cat.on_cmd(cmds=["房间", "room"], states="room")
async def room_info():
    '''展示房间内成员列表'''
    game = cat.get_data(Game)
    await cat.send(game.room_info)


@cat.on_cmd(cmds=["投票情况", "info"], states="play")
async def play_info():
    '''展示投票情况'''
    game = cat.get_data(Game)
    await cat.send(game.players_state)
    await cat.send(game.vote_info)


@cat.on_cmd(cmds=["vote", "投票"], states="play")
async def vote():
    '''请at你要投票的对象，一旦投票无法更改'''
    game = cat.get_data(Game)
    if not cat.current.arg:
        return

    uid = cat.params.nums[0]

    # 投票
    f, info = game.vote(cat.current.sender_id, uid)
    await cat.send(info)

    # 投票失败
    if not f:
        return

    # 投票是否结束
    if not game.voted_end:
        return

    # 结算时公布投票结果
    f, info = game.kickout()
    await cat.send(info)

    # 出现平局
    if not f:
        return

    # 成功踢出一人，判断游戏是否结束
    f, info = game.check_end()
    if not f:
        await cat.send("游戏继续！")
        return

    # 展示结果
    await cat.send(game.conclude())
    await cat.send(info)

    # 返回房间
    cat.state = "room"
    await cat.send("已回到房间，可发送start开始下一局")



'''
    +1s 
'''
from asyncio import sleep
from random import choice
from ayaka import AyakaCat

cat = AyakaCat("加一秒")
cat.help = '''
每人初始时间值为0
每有3个不同的人执行一次或若干次加1，boss就会完成蓄力，吸取目前时间值最高的人的时间，如果有多人，则都被吸取
boss时间值>=17时，游戏结束，boss将带走所有比他时间值高的人，剩余人中时间值最高的获胜，世界重启
'''

feelings = [
    "感觉身体被掏空",
    "感受到一阵空虚",
    "的时间似乎变快了",
    "似乎被夺走了什么东西"
]

restarts = [
    "世界被虚空的狂风撕碎",
    "世界在不灭的火焰中湮灭",
    "&*@）#……游戏出故障了",
    "限界的界世了越超间时的ssob"
]


def get_max(data: dict[int, int]):
    '''
        data 是保存了 uid-time 数据的字典
    '''
    # 转换为items列表，方便排序
    items = list(data.items())
    items.sort(key=lambda x: x[1], reverse=True)

    uids = []
    if not items:
        return uids

    time_max = items[0][1]
    for uid, time in items:
        if time == time_max:
            uids.append(uid)
        else:
            break

    return uids


class PlayerGroup:
    def __init__(self) -> None:
        self.data: dict[str, int] = {}

    def get_time(self, uid: str) -> int:
        '''获取uid对应的时间值，如果不存在，返回为0'''
        if uid not in self.data:
            self.data[uid] = 0
            return 0

        # 存在则返回
        return self.data[uid]

    def change_time(self, uid: str, diff: int):
        '''修改uid对应的时间值'''

        time: int = self.data.get(uid, 0)

        # 修改time
        time += diff

        self.data[uid] = time

        # 返回修改后的值
        return time

    def clear_all(self):
        self.data = {}


class Boss:
    boss_default = {
        "time": 0,
        # 记录不同的人的发言
        "uids": []
    }

    max_time = 17
    max_power = 3

    def __init__(self, player_group: PlayerGroup) -> None:
        self.data = self.boss_default
        self.player_group = player_group

    @property
    def time(self) -> int:
        return self.data["time"]

    @time.setter
    def time(self, v: int):
        self.data["time"] = v

    @property
    def power(self):
        return len(self.data["uids"])

    def clear_power(self):
        self.data["uids"] = []

    def add_power(self, uid: int):
        # 防止重复
        uids: list = self.data["uids"]
        if uid in uids:
            return
        uids.append(uid)
        self.data["uids"] = uids

    def kill(self, data: dict):
        '''
            data 是保存了 uid-time 数据的字典
        '''
        # 发动攻击，清除power
        self.clear_power()

        # 记录吸取情况
        uids = get_max(data)

        # 吸取时间
        cnt = 0
        for uid in uids:
            i = choice([1, 1, 1, 2, 2, 3])
            cnt += i
            self.player_group.change_time(uid, -i)
        self.time += cnt

        # 告知吸取情况
        return uids

    @property
    def state(self):
        info = f"boss目前的时间：{self.time}/{self.max_time}\nboss目前的能量：{self.power}/{self.max_power}"
        if self.power >= self.max_power - 1:
            info += "\nboss即将发动攻击！"
        if self.time >= self.max_time - 1:
            info += "\nboss的时间即将到达顶峰！"
        return info


class Game:
    def __init__(self) -> None:
        self.reset()

    def reset(self) -> None:
        self.player_group = PlayerGroup()
        self.boss = Boss(self.player_group)

    def get_over_players(self):
        boss_time = self.boss.time
        data = self.player_group.data
        # 时间值>boss的人
        uids = [uid for uid, time in data.items() if time > boss_time]
        return uids

    def get_winners(self):
        boss_time = self.boss.time
        data = self.player_group.data
        # 时间值小于等于boss的人
        data = {uid: time for uid, time in data.items() if time <= boss_time}
        return get_max(data)


@cat.on_cmd(cmds="加一秒")
async def cat_start():
    '''启动游戏'''
    game = cat.get_data(Game)
    if not game.boss:
        game.reset()
    await cat.wakeup()
    await cat.send_help()


@cat.on_cmd(cmds=["exit", "退出"], states="*")
async def cat_close():
    '''退出游戏（数据保留）'''
    await cat.send("数据已保存")
    await cat.rest()


@cat.on_cmd(cmds="我的", states="idle")
async def inquiry():
    '''查看你目前的时间'''
    game = cat.get_data(Game)
    time = game.player_group.get_time(cat.current.sender_id)
    await cat.send(f"[{cat.current.sender_name}]目前的时间：{time}")


@cat.on_cmd(cmds="boss", states="idle")
async def inquiry_boss():
    '''查看boss的时间和能量'''
    game = cat.get_data(Game)
    await cat.send(game.boss.state)


@cat.on_cmd(cmds="全部", states="idle")
async def inquiry_all():
    '''查看所有人参与情况，以及boss的时间和能量'''
    game = cat.get_data(Game)
    # boss
    info = game.boss.state

    # 所有人
    data = game.player_group.data

    if not data:
        await cat.send(info)
        return

    # 查找名字
    users = await cat.get_users()
    user_data = {u.id: u.name for u in users}

    for uid, time in data.items():
        info += f"\n[{user_data[uid]}]目前的时间：{time}"
    await cat.send(info)


@cat.on_cmd(cmds=["加一", "加一秒"], states="idle")
async def plus():
    '''让你的时间+1'''
    game = cat.get_data(Game)

    # 玩家
    time = game.player_group.change_time(cat.current.sender_id, 1)
    await cat.send(f"[{cat.current.sender_name}]的时间增加了！目前为：{time}")

    # boss
    game.boss.add_power(cat.current.sender_id)
    await cat.send(game.boss.state)

    # boss 能量不够
    if game.boss.power < game.boss.max_power:
        return

    # boss 攻击
    uids = game.boss.kill(game.player_group.data)

    await cat.send("boss发动了攻击...")
    await sleep(2)
    await cat.send("...")
    await sleep(2)

    if not uids:
        await cat.send("无事发生")
        return

    # 查找名字
    users = await cat.get_users()
    user_data = {u.id: u.name for u in users}

    # 告知被攻击情况
    items = [f"[{user_data[uid]}] {choice(feelings)}" for uid in uids]
    info = "\n".join(items)
    await cat.send(info)
    await inquiry_all()

    # boss 时间不够
    if game.boss.time < game.boss.max_time:
        return

    # 游戏结束
    await sleep(2)
    await cat.send(f"boss的时间超越了世界的界限，{choice(restarts)}...")
    await sleep(2)
    await cat.send("...")
    await sleep(2)

    # 带走时间过高的玩家
    uids = game.get_over_players()
    items = [f"[{user_data[uid]}]" for uid in uids]

    if items:
        info = f"boss带走了所有时间高于它的玩家：" + "、".join(items)
        await cat.send(info)

    # 告知胜利者
    uids = game.get_winners()
    items = [f"[{user_data[uid]}]" for uid in uids]

    if not items:
        info = "没有人"
    else:
        info = "、".join(items)

    info = "在上一个世界中：" + info + "是最终的赢家！"
    await cat.send(info)

    game.player_group.clear_all()
    game.boss.time = 0



'''
    成语查询
'''
from random import sample
from ayaka import AyakaCat, load_data_from_file
from .data import downloader

cat = AyakaCat("成语查询")
cat.help = '''
有效提高群文学氛围

数据来源：成语大全（20104条成语数据）
'''

search_dict = {}


@downloader.on_finish
async def finish():
    global search_dict
    path = downloader.BASE_DIR / "成语词典.json"
    search_dict = load_data_from_file(path)


async def show_word(word: str):
    if word in search_dict:
        info = word + "\n" + search_dict[word]
        await cat.send(info)
    else:
        await cat.send("没有找到相关信息")


@cat.on_cmd(cmds="搜索成语")
async def handle_3():
    '''搜索所有相关的成语，可输入多个关键词更准确'''
    args = [arg for arg in cat.current.args if isinstance(arg, str)]

    if not args:
        await cat.send("没有输入关键词")
        return

    words = []
    for _word in search_dict:
        for arg in args:
            if arg not in _word:
                break
        else:
            words.append(_word)

    if not words:
        await cat.send("没有找到相关信息")
        return

    n = len(words)

    if n > 3:
        infos = [
            f"搜索关键词：{args}",
            f"共找到{n}条相关信息"
        ]
        if n > 100:
            infos.append("数量过多，仅展示随机抽取的100条")
            words = sample(words, 100)
        infos.append("\n".join(words))
        await cat.send_many(infos)
    else:
        for word in words:
            await show_word(word)



'''24点，以及拓展版48点'''
import re
from random import choice
from pydantic import BaseModel
from loguru import logger
from ayaka import AyakaCat, load_data_from_file
from .bag import get_money
from .data import downloader, config

cat = AyakaCat("24点")
cat.help = '''
加、减、乘、除、次方，5种运算符可用
给出4个1-9范围内的数字，请通过以上运算符算出24点
同时还有48点等其他模式可选，欢迎挑战
'''


questions_bin: dict[int, dict[str, list[str]]] = {}


class Cache(BaseModel):
    n: int = 0
    nums: list[int] = []
    solutions: list[str] = []

    @property
    def question(self):
        return " ".join(str(n) for n in self.nums)


@downloader.on_finish
async def finish():
    path = downloader.BASE_DIR / "24点"
    for p in path.iterdir():
        try:
            questions_bin[int(p.stem)] = load_data_from_file(p)
        except:
            pass


cat.set_rest_cmds(cmds=["退出", "exit"])


@cat.on_cmd(cmds="24点")
async def cat_start():
    '''唤醒猫猫，开始游戏'''
    await cat.wakeup()
    await cat.send_help()
    await set_mode_menu()


@cat.on_cmd(cmds="切换模式", states="*")
async def set_mode_menu():
    items = ["切换模式："]
    items.extend(f"- {n}" for n in questions_bin)
    items.append("请输入")
    await cat.send("\n".join(items))
    cat.state = "切换模式"


@cat.on_text(states="切换模式")
async def set_mode():
    '''更多挑战'''
    if not cat.current.nums or cat.current.nums[0] not in questions_bin:
        await cat.send("输入不合法")
        return

    cache = cat.get_data(Cache)
    cache.n = cat.current.nums[0]
    await cat.send(f"切换模式为 {cache.n}点")
    cat.state = "计算"
    await select_question()


@cat.on_cmd(cmds=["出题", "next"], states="计算")
async def select_question():
    '''出题'''
    cache = cat.get_data(Cache)

    data = questions_bin[cache.n]
    k, cache.solutions = choice(list(data.items()))
    cache.nums = [int(n) for n in k.split(" ")]

    await show_question()


@cat.on_cmd(cmds=["题目", "question"], states="计算")
async def show_question():
    '''展示题目'''
    cache = cat.get_data(Cache)
    await cat.send(cache.question)
    await cat.send(f"TIPS：本题至少有{len(cache.solutions)}种答案（使用不同的运算符）")


@cat.on_cmd(cmds=["答案", "answer"], states="计算")
async def show_solutions():
    '''展示答案'''
    cache = cat.get_data(Cache)
    solutions = cache.solutions

    info = "\n".join(solutions)
    await cat.send(info)
    await select_question()


@cat.on_text(states="计算")
async def calculate_exp():
    '''请使用正确的表达式，例如 (1+2)*(3+3)'''
    cache = cat.get_data(Cache)
    exp = cat.current.arg

    try:
        exp = pre_check(exp, cache.nums)
    except Exception as e:
        await cat.send(str(e))
        return

    try:
        r = calc(exp)
    except Exception as e:
        await cat.send(str(e))
        return

    await cat.send(f"{exp}={r}")

    if abs(r - cache.n) > 0.0001:
        await cat.send("错误")
        return

    await cat.send("正确！")
    reward = config.calculate_reward
    money = get_money(group_id=cat.current.session_id,
                      user_id=cat.current.sender_id)
    money.value += reward
    await cat.send(f"奖励{reward}金")
    await select_question()

# ---- 预检查：对字符串做初步的筛查和简单的纠正 ----


def check_len(exp: str):
    '''拒绝长度异常的表达式'''
    return len(exp) <= 20


def pre_correct(exp: str):
    '''初步矫正表达式：移除空格，纠正操作符'''
    exp = exp.strip().replace("\\", "/").replace(" ", "")
    exp = exp.replace("x", "*").replace("^", "**")
    exp = re.sub(r"[（{【\[]", "(", exp)
    exp = re.sub(r"[）}】\]]", ")", exp)
    return exp


def check_brackets_close(exp: str):
    '''检查括号配对是否闭合'''
    cnt = 0
    for t in exp:
        if t == "(":
            cnt += 1
        elif t == ")":
            cnt -= 1
            if cnt < 0:
                return False
    return cnt == 0


def check_op(exp: str):
    '''检测无效操作符'''
    patt = re.compile(r"\*\*|\*|\+|-|/|\(|\)|\d+")
    return not patt.sub("", exp)


def split_exp(exp: str):
    '''分割表达式'''
    patt = re.compile(r"\*\*|\*|\+|-|/|\(|\)|\d+")
    ts: list[int | str] = []
    for t in patt.findall(exp):
        if t not in ["(", ")", "+", "-", "*", "/", "**"]:
            t = int(t)
        ts.append(t)
    return ts


def check_num(ts: list[str | int], nums: list[int]):
    '''检查是否使用指定数字'''
    nums_copy = [n for n in nums]
    for t in ts:
        if isinstance(t, int):
            if t not in nums_copy:
                return False
            nums_copy.remove(t)
    return not nums_copy


def check_and_correct_op(ts: list[str | int]):
    '''补全缺失的*，检查错误组合'''

    _ts = [ts[0]]
    last = ts[0]

    for t in ts[1:]:
        # 数字 + (，补全*
        if isinstance(last, int) and t == "(":
            _ts.append("*")

        # (只能 + 数字或(
        elif last == "(" and isinstance(t, str) and t != "(":
            return

        # ) + (或数字，补全*
        elif last == ")" and (isinstance(t, int) or t == "("):
            _ts.append("*")

        # 其他操作符必定不 + )
        elif last in ["+", "-", "*", "/", "**"] and t == ")":
            return

        _ts.append(t)
        last = t
    return _ts


def pre_check(exp, nums):
    '''预检查，并简单矫正一些错误，返回纠正后的表达式，抛出错误'''
    if not check_len(exp):
        raise Exception("表达式过长")

    exp = pre_correct(exp)

    if not check_brackets_close(exp):
        raise Exception("括号未闭合")

    if not check_op(exp):
        raise Exception("无效操作符")

    ts = split_exp(exp)

    if not check_num(ts, nums):
        raise Exception("没有使用指定数字")

    ts = check_and_correct_op(ts)
    if not ts:
        raise Exception("错误的操作符组合")

    return "".join(str(t) for t in ts)

# ---- 计算合法格式的中缀表达式 ----


def mid2post(ts: list[str | int]):
    '''中缀表达式转后缀表达式'''
    ts.append("#")
    stack = ["#"]
    rs: list[str | int] = []

    _in = {
        "#": 0,
        "(": 1,
        "+": 3, "-": 3,
        "*": 5, "/": 5,
        "**": 7,
    }
    _out = {
        "#": 0,
        ")": 1,
        "+": 2, "-": 2,
        "*": 4, "/": 4,
        "**": 6,
        "(": 8,
    }

    for t in ts:
        # 数字直接计入结果
        if isinstance(t, int):
            rs.append(t)
            continue

        # 操作符判断优先级入栈
        d = _out[t] - _in[stack[-1]]

        # 若 out < in，则内部操作符优先级更高，出栈，计入结果，且循环判断
        # 注意：栈不可能通过此条件弹出至空，因为 # 对应 最小in = 最小out = 0
        while d < 0:
            rs.append(stack.pop())
            d = _out[t] - _in[stack[-1]]

        # 若 out > in，则外部操作符优先级更高，入栈
        if d > 0:
            stack.append(t)

        # 若 out == in，则优先级相同，出栈，且不计入结果
        else:
            stack.pop()

    return rs


def base_calc(n1: float, n2: float, op: str):
    '''计算数值，出错时抛出异常'''
    logger.debug(f"基本运算单元 {n1}{op}{n2}")

    if op == "*":
        return n1*n2

    # 除0自动抛出异常
    if op == "/":
        return n1 / n2

    if op == "+":
        return n1 + n2

    if op == "-":
        return n1 - n2

    if op == "**":
        # 阻断大指数运算
        if n1 > 100 or n2 > 100:
            raise
        return n1 ** n2

    # 未知操作符
    raise


def calc_post(ts: list[str | int]):
    '''计算后缀表达式'''
    ops: list[str] = []
    nums: list[float] = []
    for t in ts:
        if isinstance(t, str):
            ops.append(t)
        else:
            nums.append(float(t))

        if len(nums) >= 2 and ops:
            n2 = nums.pop()
            n1 = nums.pop()
            op = ops.pop()
            n = base_calc(n1, n2, op)

            # 中断过大或过小的运算
            if abs(n) > 10000 or abs(n) < 0.00001:
                raise

            nums.append(n)

    return nums[0]


def calc(exp):
    '''计算表达式，返回计算的结果，抛出异常'''
    ts = split_exp(exp)
    ts = mid2post(ts)

    logger.debug(f"后缀表达式 {ts}")

    try:
        v = calc_post(ts)
    except:
        raise Exception(choice(["什么鬼啦！", "你正常点，我害怕", "这，这不对吧", "寄"]))

    return v


@cat.on_cmd(cmds="测试24点", always=True)
async def test_exp():
    '''<表达式>'''
    exp = str(cat.current.arg)
    try:
        r = calc(exp)
    except Exception as e:
        await cat.send(str(e))
        return

    await cat.send(f"{exp}={r}")



from random import sample
from time import time
from pydantic import BaseModel, Field
from ayaka import AyakaCat, AyakaUserDB, AyakaGroupDB, AyakaDB, load_data_from_file
from .bag import get_money, Money
from .data import downloader, config

cat = AyakaCat("文字税")
cat.help = '''知识付费（doge

注意：只有买了文字的人之间才会相互交税，其他人不受影响'''

words: list[str] = []


@downloader.on_finish
async def finish():
    path = downloader.BASE_DIR / "文字税.txt"
    words.extend(load_data_from_file(path))


class UserWord(AyakaUserDB):
    __table_name__ = "word_tax"
    word: str = Field(extra=AyakaDB.__primary_key__)
    uname: str = ""
    time: int = 0


class GroupWord(AyakaGroupDB):
    __table_name__ = "word_tax_group"
    words: str = ""
    time: int = 0


class GroupMarket(BaseModel):
    time: int = 0
    words: list[str] = []
    users: list[UserWord] = []
    first: bool = True

    def __init__(self, **data) -> None:
        super().__init__(**data)
        self.load(cat.current.session_id)

    def load(self, group_id):
        if self.first:
            self.first = False
            self.users = UserWord.select_many(group_id=group_id)
            gw = GroupWord.select_one(group_id=group_id)
            self.time = gw.time
            self.words = [u for u in gw.words]

    def refresh(self, group_id):
        self.first = False
        self.time = int(time())
        self.words = sample(words, 20)
        gw = GroupWord.select_one(group_id=group_id)
        gw.words = "".join(self.words)
        gw.time = self.time
        self.users = []
        UserWord.delete(
            group_id=group_id
        )

    def is_open(self):
        return int(time()) - self.time <= config.word_tax.open_duration

    def open_time_left(self):
        return max(0, config.word_tax.open_duration + self.time - int(time()))

    def is_valid(self):
        return int(time()) - self.time <= config.word_tax.valid_duration

    def buy(self, group_id, user_id, uname):
        UserWord.delete(
            group_id=group_id,
            user_id=user_id,
        )
        self.users = [
            u for u in self.users
            if u.group_id != group_id or u.user_id != user_id
        ]
        words = sample(self.words, 3)
        users = [
            UserWord(
                group_id=group_id,
                user_id=user_id,
                word=word,
                time=self.time,
                uname=uname
            )
            for word in words
        ]
        UserWord.insert_many(users)
        self.users.extend(users)
        return words

    def check(self, msg: str, user_id: int):
        check_dict: dict[str, list[UserWord]] = {}
        for u in self.users:
            if u.user_id == user_id:
                break
        else:
            # 排除没抽的人
            return {}

        for u in self.users:
            if u.user_id == user_id:
                continue
            if u.word not in check_dict:
                check_dict[u.word] = [u]
            else:
                check_dict[u.word].append(u)

        _check_dict: dict[str, list[UserWord]] = {}
        for m in msg:
            if m not in _check_dict and m in check_dict:
                _check_dict[m] = check_dict[m]
        return _check_dict

    def get_words(self, user_id: int):
        return [u.word for u in self.users if u.user_id == user_id]


@cat.on_cmd(cmds="文字市场")
async def open_word_market():
    '''查看文字市场'''
    market = cat.get_data(GroupMarket)
    info = "，".join(market.words)
    if market.is_open():
        dd = "开放"
    else:
        dd = "关闭"
    await cat.send(f"市场已{dd}，本轮文字池为\n{info}\n剩余持续时间{market.open_time_left()}s")


@cat.on_cmd(cmds=["开放文字市场", "刷新文字市场"])
async def open_word_market():
    '''刷新文字市场，开放福袋购买'''
    user = await cat.get_user(cat.current.sender_id)
    if user.role not in ["owner", "admin"]:
        await cat.send("请联系管理员开放市场，您没有权限")
        return

    market = cat.get_data(GroupMarket)
    market.refresh(cat.current.session_id)
    info = "，".join(market.words)
    await cat.send(f"市场已开放，持续{config.word_tax.open_duration}s，本轮文字池为\n{info}")


@cat.on_cmd(cmds="购买文字")
async def buy_words():
    '''花费金钱购买一次福袋，获得3个随机文字'''
    market = cat.get_data(GroupMarket)
    if not market.is_open():
        await cat.send("市场未开放，请联系管理员开放市场后再购买")
    else:
        money = get_money(cat.current.session_id, cat.current.sender_id)
        money.value -= config.word_tax.buy_price
        words = market.buy(
            cat.current.session_id, cat.current.sender_id, cat.current.sender_name)
        info = "，".join(words)
        await cat.send(f"[{cat.current.sender_name}] 花费{config.word_tax.buy_price}金，购买了文字 {info}")


@cat.on_cmd(cmds="我的文字")
async def buy_words():
    '''查看自己的文字'''
    market = cat.get_data(GroupMarket)
    words = market.get_words(cat.current.sender_id)
    info = "，".join(words)
    await cat.send(f"[{cat.current.sender_name}]当前拥有 {info}")


@cat.on_cmd(cmds="所有文字")
async def buy_words():
    '''查看所有人的文字'''
    market = cat.get_data(GroupMarket)
    if market.is_valid():
        data: dict[str, str] = {}
        for u in market.users:
            if u.uname not in data:
                data[u.uname] = ""
            data[u.uname] += u.word
        items = [f"[{k}] {v}" for k, v in data.items()]
        await cat.send_many(items)
    else:
        await cat.send("这一轮文字税尚未开始")


@cat.on_text()
async def get_tax():
    '''计算税收'''
    market = cat.get_data(GroupMarket)
    if market.is_open():
        return

    if not market.is_valid():
        return

    msg = cat.current.arg
    check_dict = market.check(msg, cat.current.sender_id)
    if not check_dict:
        return

    user_moneys: dict[int, Money] = {}
    for users in check_dict.values():
        for u in users:
            if u.user_id not in user_moneys:
                user_moneys[u.user_id] = get_money(u.group_id, u.user_id)
    user_moneys[cat.current.sender_id] = get_money(
        cat.current.session_id, cat.current.sender_id)

    for w in check_dict.keys():
        msg = msg.replace(w, f"[{w}]")
    infos = [msg]

    for w, users in check_dict.items():
        t = int(config.word_tax.tax / len(users))
        tt = t*len(users)
        user_moneys[cat.current.sender_id].value -= tt
        for u in users:
            user_moneys[u.user_id].value += t
        names = [u.uname for u in users]
        infos.append(f"[{w}] 所有者：{names}，您为此字付费{tt}金")

    if config.word_tax.tax_notice:
        await cat.send_many(infos)



import json
import asyncio
from typing import Awaitable, Callable
from pathlib import Path
from pydantic import BaseModel
from loguru import logger
from ayaka import resource_download_by_res_info, ResInfo, resource_download, AyakaConfig, bridge


class WordTaxConfig(BaseModel):
    tax: int = 100
    buy_price: int = 1000
    open_duration: int = 300
    valid_duration: int = 86400
    tax_notice: bool = True


class PrayConfigItem(BaseModel):
    reward: int = 0
    weight: int = 0


class Config(AyakaConfig):
    __config_name__ = "ayaka_game"
    auto_update: bool = True
    calculate_reward: int = 1000
    checkin_reward: int = 10000
    dragon_reward: int = 1000
    word_tax: WordTaxConfig = WordTaxConfig()
    pray: list[PrayConfigItem] = [
        PrayConfigItem(reward=66, weight=10),
        PrayConfigItem(reward=666, weight=50),
        PrayConfigItem(reward=6666, weight=30),
        PrayConfigItem(reward=66666, weight=5),
        PrayConfigItem(reward=-66666, weight=5),
    ]


config = Config()
AUTHOR = "bridgeL"
REPO = "ayaka_games"
BRANCH = "master"


class Downloader:
    RESINFO_URL = f"https://ghproxy.com/https://raw.githubusercontent.com/{AUTHOR}/{REPO}/{BRANCH}/res_info.json"
    '''资源信息下载地址'''

    BASE_DIR = Path("data", "ayaka_games")
    '''本地文件地址'''

    def __init__(self) -> None:
        self.funcs: list[Callable[[], Awaitable]] = []

    async def download_data(self):
        '''更新数据'''
        logger.info("检查资源文件...")

        if config.auto_update:
            try:
                data = await resource_download(self.RESINFO_URL)
            except:
                logger.warning("目前无法获取资源目录，这意味资源数据无法同步为最新版本，尽管如此，仍可加载本地的资源文件")
            else:
                data = json.loads(data)
                res_info = ResInfo(**data)
                await resource_download_by_res_info(res_info, self.BASE_DIR)

        ts = [asyncio.create_task(func()) for func in self.funcs]
        await asyncio.gather(*ts)

        logger.info("资源文件准备完毕")

    def on_finish(self, func: Callable[[], Awaitable]):
        '''检查文件更新结束后执行的异步回调'''
        self.funcs.append(func)
        return func


downloader = Downloader()


@bridge.on_startup
async def download():
    '''检查资源更新'''
    asyncio.create_task(downloader.download_data())



from asyncio import sleep
from random import randint
from ayaka import AyakaCat, AyakaGroupDB, AyakaUserDB
from .bag import get_money

god_names = ['欢愉', '悼亡', '深渊', '智慧']

cat = AyakaCat('mana')
cat.help = '''
===== m a n a =====
欢愉、悼亡、深渊、智慧
===== ======= =====
'''


class ManaGod(AyakaGroupDB):
    __table_name__ = "mana_god"
    name: str = "欢愉"
    power: int = 1
    cnt: int = 0
    mana: int = 0

    def say(self):
        words = {
            '欢愉': ['命途对你报之一笑', '命途轻咬你的耳根', '命途正引诱你堕入其中'],
            '悼亡': ['命途中，亡魂在悲叹', '命途中充斥着悲伤', '命途默然无声地崩解着'],
            '深渊': ['命途静静凝视着你'],
            '智慧': ['命途终极的智慧，令你感到畏惧', '命途看穿了你的企图', '命途对你的愚蠢感到无趣'],
        }

        cs = {
            '欢愉': '♥',
            '悼亡': '♠',
            '深渊': '♣',
            '智慧': '♦',
        }

        # ps = ['魔法师','女祭司','皇后','皇帝','教皇','恋人','战车','力量','隐士','命运之轮','正义','倒吊人','死神']

        rs = words[self.name]
        r = rs[randint(0, len(rs)-1)]
        c = cs[self.name]
        return f"{r} {c} {self.power}"


def get_mana_god(group_id: int,):
    return ManaGod.select_one(group_id=group_id)


class UserMana(AyakaUserDB):
    __table_name__ = "user_mana"
    mana: int = 10


def get_user_mana(group_id: int, user_id: int):
    return UserMana.select_one(
        group_id=group_id,
        user_id=user_id
    )


def change_god(god_name: str, god: ManaGod):
    god.name = god_name
    god.power = randint(1, 13)
    god.cnt = 0
    god.mana = 0


def happy(god: ManaGod, mana: int):
    if mana < -100:
        change_god(god_names[randint(0, 3)], god)
        return mana*2, "命途为你的愚行而惊讶"

    power = god.power

    god.mana += abs(mana)
    if god.mana >= 8:
        god.power += int(god.mana/8)
        god.mana = 0

    god.cnt += 1
    if god.cnt >= 3:
        god.power += 1
        god.cnt = 0

    if god.power > 13:
        change_god(god_names[randint(0, 3)], god)

    if power <= 3:
        return 0, "命途对你毫无兴趣"
    if power <= 6:
        return 2*(power-3), "命途微微舒展身姿"
    if power <= 10:
        return mana*(1+(power-6)/5), "命途对你兴趣颇丰"
    return mana*(power-9), "命途向你展示了星空的无限隐秘"


def sorrow(god: ManaGod, mana: int):
    if mana < -100:
        change_god(god_names[randint(0, 3)], god)
        return mana*2, "命途为你的愚行而哀伤"

    power = god.power

    god.mana += abs(mana)
    if god.mana >= power:
        god.power += int(god.mana/power)
        god.mana = 0

    god.cnt += 1
    if god.cnt >= 10:
        god.power += 1
        god.cnt = 0

    if god.power > 13:
        change_god(god_names[randint(0, 3)], god)

    if power <= 5:
        return mana, "命途对你悲叹"
    if power <= 10:
        return mana + power - 5, "命途对你怜悯"
    return - power, "命途对你哀悼"


def abyss(god: ManaGod, mana: int):
    if mana < -100:
        change_god(god_names[randint(0, 3)], god)
        return mana*2, "命途为你的愚行而喜悦"

    power = god.power

    if mana % 3 == 2:
        god.power -= 1
    else:
        god.power += abs(mana) % 3

    god.cnt += 1
    if god.cnt >= 12:
        change_god(god_names[randint(0, 3)], god)

    if god.power > 13:
        change_god('欢愉', god)

    if god.power <= 0:
        change_god('悼亡', god)

    if power % 3 == 0 and randint(0, 2):
        return mana * power / 3, "当你理解了命途，你也就成为了命途"
    return mana * (1 - power/6), "命途贪婪的吞噬了一切"


def wise(god: ManaGod, mana: int):
    if mana < -100:
        change_god(god_names[randint(0, 3)], god)
        return mana*2, "..."

    power = god.power
    reward = int(god.mana / 3)

    god.mana += abs(mana)*2 - reward
    if power <= 7:
        god.mana += power*3
    else:
        god.mana += (14 - power)*3

    god.power = int(god.mana / 3) + 1

    god.cnt += 1
    if god.cnt >= 12:
        change_god(god_names[randint(0, 3)], god)

    if god.power > 13 or god.power <= 0:
        change_god(god_names[randint(0, 3)], god)

    return reward, "宇宙的奥秘在命途中盘旋"


cat.set_wakeup_cmds(cmds=["mana", "玛娜"])
cat.set_rest_cmds(cmds=["exit", "退出"])


@cat.on_cmd(cmds=['divine', '占卜'], states="idle")
async def pray():
    '''花费1玛娜，祈求神的回应'''
    usermana = get_user_mana(cat.current.session_id, cat.current.sender_id)
    god = get_mana_god(cat.current.session_id)

    name = cat.current.sender_name

    mana = usermana.mana
    if mana <= 0:
        await cat.send(f"[{name}] 没有玛娜")
        return

    usermana.mana -= 1

    await cat.send(f"[{name}] 花费了 1玛娜，聆听星辰的呓语")
    await sleep(1)
    await cat.send(f"...")
    await sleep(1)
    await cat.send(f"...")
    await sleep(1)

    await cat.send(god.say())


@cat.on_cmd(cmds=['pray', '祈祷'], states="idle")
async def handle():
    '''n 花费n玛娜，感受神的呼吸'''
    usermana = get_user_mana(cat.current.session_id, cat.current.sender_id)
    god = get_mana_god(cat.current.session_id)

    if not cat.params.nums:
        await cat.send("请使用 pray <数字>")
        return

    arg = cat.params.nums[0]
    name = cat.current.sender_name

    mana = usermana.mana
    if mana < arg:
        await cat.send(f"[{name}] 只有 {mana}玛娜")
        return

    funcs = {
        '欢愉': happy,
        '悼亡': sorrow,
        '深渊': abyss,
        '智慧': wise,
    }

    old_god_name = god.name

    func = funcs[god.name]
    reward, info = func(god=god, mana=arg)
    reward = int(reward)
    await cat.send(info)
    usermana.mana += reward - arg
    mana = usermana.mana
    await cat.send(f"[{name}] 花费了 {arg}玛娜，获得了 {reward}玛娜")
    await cat.send(f"[{name}] 当前有 {mana}玛娜")

    if old_god_name != god.name:
        await cat.send("星空在惊惧中震颤，旧的命途陨落，新的命途执掌星空")
        await cat.send(god.say())


@cat.on_cmd(cmds="mana", states="idle")
async def handle():
    '''正数买入，负数卖出，0查询'''

    if not cat.params.nums:
        await cat.send("请使用 mana <数字>")
        return

    usermana = get_user_mana(cat.current.session_id, cat.current.sender_id)
    usermoney = get_money(cat.current.session_id, cat.current.sender_id)
    num = cat.params.nums[0]
    name = cat.current.sender_name

    mana = usermana.mana
    money = usermoney.value

    if num == 0:
        await cat.send(f"[{name}] 当前持有 {mana}个玛娜")
        return

    # 买mana
    if num > 0:
        action = "购买"
    else:
        action = "卖出"
        num = -num

    if action == "购买":
        if money < num*1000:
            await cat.send(f"[{name}] 只有{money}个金币")
            return
        else:
            usermoney.value -= num*1000
            money = usermoney.value
            usermana.mana += num
            mana = usermana.mana
    else:
        if mana < num:
            await cat.send(f"[{name}] 只有{mana}个玛娜")
            return
        else:
            usermoney.value += num*1000
            money = usermoney.value
            usermana.mana -= num
            mana = usermana.mana

    await cat.send(f"[{name}] {action}玛娜成功，当前持有 {money}个金币，{mana}个玛娜")



from random import randint
from ayaka import AyakaCat
from .bag import get_money
from .data import config

cat = AyakaCat("pray")
cat.help = '祈福'

all_wegiht = 0
for item in config.pray:
    all_wegiht += item.weight


def get_diff():
    target = randint(0, all_wegiht-1)
    print(target)

    sum = 0
    for item in config.pray:
        if sum <= target and sum+item.weight > target:
            return item.reward
        sum += item.weight

    raise


@cat.on_cmd(cmds=["pray", "祈祷"])
async def handle():
    '''为群里随机一人（除了自己）祈祷随机金币'''
    nodes = await cat.get_users()

    # 排除自己
    nodes = [node for node in nodes if node.id != cat.current.sender_id]

    # 找到受害人
    node = nodes[randint(0, len(nodes)-1)]
    uid = node.id
    name = node.name

    prayer_name = cat.current.sender_name

    diff = get_diff()
    money = get_money(cat.current.session_id, uid)
    money.value += diff

    await cat.send(f"[{prayer_name}]的祈祷，让[{name}]获得 {diff}金")
    if diff < 0:
        await cat.send(f"反转了，[{name}]损失 {-diff}金")
    await cat.send(f"[{name}] 现在持有 {money.value}金")



'''
    抢30
'''
from asyncio import sleep
from random import randint
from ayaka import AyakaCat

cat = AyakaCat("抢30")
cat.help = '''
至少2人游玩，每人开局10个筹码，每轮自增1个
每轮从52张牌中抽取一张（不放回），所有人按轮次使用筹码参与竞价，出价最高的人获得此牌对应的点数
重复多轮，谁的点数先达到30点，谁获胜
52张牌直到用尽后才会洗牌，且只要不退出游戏，下局的牌库将继承上局
'''


class Player:
    def __init__(self, uid: int, name: str) -> None:
        self.uid = uid
        self.name = name

        self.money = 0
        self.quote = 0
        self.score = 0
        self.win_cnt = 0
        self.first = False

    def quote_win(self, card: int):
        self.money -= self.quote
        self.score += card


def shuffle(array: list):
    result = []
    while array:
        i = randint(0, len(array)-1)
        result.append(array.pop(i))
    return result


class Game:
    def __init__(self) -> None:
        self.reset()

    def reset(self):
        self.players: list[Player] = []
        self.cards = []
        self.first = -1
        self.i = 0
        self.card = 0
        self.last_quote = -1

    @property
    def player_cnt(self):
        return len(self.players)

    def get_player(self, uid: int):
        for p in self.players:
            if p.uid == uid:
                return p

    def join(self, uid: int, name: str):
        p = self.get_player(uid)
        if p:
            return False, f"[{p.name}] 已在房间中"
        p = Player(uid, name)
        self.players.append(p)
        return True, f"[{p.name}] 加入房间"

    def leave(self, uid: int):
        p = self.get_player(uid)
        if not p:
            return False, f"({uid}) 不在房间中"
        self.players.remove(p)
        return True, f"[{p.name}] 离开房间"

    def refresh_cards(self):
        if not self.cards:
            self.cards = shuffle([i+1 for i in range(13) for j in range(4)])

    @property
    def player_now(self):
        i = (self.first+self.i) % self.player_cnt
        return self.players[i]

    def refresh_card(self):
        self.refresh_cards()
        self.card = self.cards.pop()

    def start(self):
        if self.player_cnt < 2:
            return False, "至少需要2人才能开始游戏"

        # 初始化状态
        for p in self.players:
            p.money = 9
            p.quote = -1
            p.score = 0
            p.first = False

        return True, "游戏开始"

    def round_begin(self):
        for p in self.players:
            p.quote = -1
            p.money += 1
            p.first = False

        self.first = (self.first+1) % self.player_cnt
        self.i = 0
        self.last_quote = -1
        self.refresh_card()
        self.player_now.first = True

    @property
    def round_is_end(self):
        return self.i >= self.player_cnt

    def quote(self, uid: int, quote: int):
        # 参数校验
        if not self.get_player(uid):
            return False, f"({uid}) 没有加入游戏"

        # 没轮到你
        p = self.player_now
        if p.uid != uid:
            return False, f"没轮到你"

        if quote != 0 and quote <= self.last_quote:
            return False, f"至少大于{self.last_quote}，或者0放弃"

        if quote > p.money:
            return False, "你没有那么多钱"

        p.quote = quote
        self.last_quote = quote
        self.i += 1
        return True, f"[{p.name}] 报价 [{quote}]"

    def settle(self):
        ps = [p for p in self.players]

        # 取最高者
        ps.sort(key=lambda p: p.quote, reverse=True)
        p = ps[0]

        # 庄家赚了
        if p.quote == 0:
            p = self.players[self.first]

        # 赢得此牌
        p.quote_win(self.card)
        return f"[{p.name}] 赢得此牌"

    @property
    def winner(self):
        for p in self.players:
            if p.score >= 30:
                return p

    @property
    def card_info(self):
        return f"当前牌 {self.card}"

    @property
    def player_info(self):
        items = []
        items.append("报价 筹码 点数 玩家名")
        for p in self.players:
            if p.quote >= 0:
                quote = p.quote
            else:
                quote = "无"

            if p == self.player_now:
                prefix = ">"
            else:
                prefix = "-"

            items.append(
                f"{prefix}  {quote}  {p.money}  {p.score}  [{p.name}]")
        return "\n".join(items)

    @property
    def room_info(self):
        items = ["房间成员："]
        for p in self.players:
            if p.win_cnt:
                win = f"获胜{p.win_cnt}局"
            else:
                win = ""
            items.append(f"[{p.name}] {win}")
        return "\n".join(items)


@cat.on_cmd(cmds="抢30")
async def cat_entrance():
    '''打开游戏'''
    await cat.wakeup("room")
    await cat.send_help()

    game = cat.get_data(Game)
    game.reset()
    f, info = game.join(cat.current.sender_id, cat.current.sender_name)
    await cat.send(info)

cat.set_rest_cmds(cmds=["退出", "exit"])
cat.on_cmd(cmds=["help", "帮助"], states="*")(cat.send_help)


@cat.on_cmd(cmds=["join", "加入"], states="room")
async def join():
    '''加入房间'''
    game = cat.get_data(Game)
    f, info = game.join(cat.current.sender_id, cat.current.sender_name)
    await cat.send(info)


@cat.on_cmd(cmds=["leave", "离开"], states="room")
async def leave():
    '''离开房间'''
    game = cat.get_data(Game)
    f, info = game.leave(cat.current.sender_id)
    await cat.send(info)

    if f and game.player_cnt == 0:
        await cat.rest()


@cat.on_cmd(cmds=["start", "begin", "开始"], states="room")
async def start():
    '''开始游戏'''
    game = cat.get_data(Game)
    f, info = game.start()
    await cat.send(info)

    # 启动失败
    if not f:
        return

    cat.state = "play"
    game.round_begin()
    await play_info()


@cat.on_cmd(cmds=["info", "信息", "房间", "room", "房间信息"], states="room")
async def room_info():
    '''展示房间信息'''
    game = cat.get_data(Game)
    await cat.send(game.room_info)


@cat.on_cmd(cmds=["info", "信息", "游戏信息"], states="play")
async def play_info():
    '''展示当前牌、所有人筹码、报价'''
    game = cat.get_data(Game)
    await cat.send(game.card_info + "\n" + game.player_info)


@cat.on_text(states="play")
async def quote():
    '''报价叫牌，要么为0，要么比上一个人高，如果全员报价为0，则本轮庄家获得该牌'''
    game = cat.get_data(Game)
    if not cat.current.nums:
        await cat.send("请输入一个数字")
        return
    num = cat.current.nums[0]
    f, info = game.quote(cat.current.sender_id, num)

    # 报价失败
    if not f:
        await cat.send(info)
        return

    await play_info()

    # 本轮是否结束
    if not game.round_is_end:
        return

    # 结算时公布报价结果
    info = game.settle()
    await cat.send(info)

    p = game.winner
    if not p:
        await sleep(2)
        game.round_begin()
        await cat.send("下一轮\n所有人筹码+1")
        await play_info()
        return

    await cat.send(game.player_info)
    await cat.send(f"[{p.name}] 获胜")
    await sleep(2)
    p.win_cnt += 1

    # 返回房间
    cat.state = "room"
    await cat.send(game.room_info)
    await cat.send("发送start开始下一局")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ayaka_games']

package_data = \
{'': ['*']}

install_requires = \
['ayaka>=0.0.0.5b1,<0.0.1.0', 'pypinyin>=0.47.1,<0.48.0']

setup_kwargs = {
    'name': 'ayaka-games',
    'version': '0.0.5b0',
    'description': 'ayaka小游戏合集',
    'long_description': '<div align="center">\n\n# Ayaka小游戏合集 - 0.0.5b0\n\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/ayaka_games)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/ayaka_games)\n![PyPI - License](https://img.shields.io/pypi/l/ayaka_games)\n![PyPI](https://img.shields.io/pypi/v/ayaka_games)\n\n开发进度 10/10\n\n**特别感谢**  [@灯夜](https://github.com/lunexnocty/Meiri) 大佬的印加宝藏\n\n</div>\n\n得益于[ayaka](https://github.com/bridgeL/ayaka)，本插件可作为如下机器人框架的插件使用\n\n- [nonebot2](https://github.com/nonebot/nonebot2)(使用[onebot11](https://github.com/nonebot/adapter-onebot)适配器)\n- [hoshino](https://github.com/Ice-Cirno/HoshinoBot)\n- [nonebot1](https://github.com/nonebot/nonebot)\n\n也可将其作为console程序离线运行\n\n## 文档\n\nhttps://bridgel.github.io/ayaka_games/\n\n## 其他\n\n本插件的前身：[nonebot_plugin_ayaka_games](https://github.com/bridgeL/nonebot-plugin-ayaka-games)\n',
    'author': 'Su',
    'author_email': 'wxlxy316@163.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/bridgeL/ayaka_games',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from ayaka import AyakaCat, AyakaUserDB

cat = AyakaCat("背包")


class Money(AyakaUserDB):
    __table_name__ = "money"
    value: int = 1000


def get_money(group_id: int, user_id: int):
    return Money.select_one(
        group_id=group_id,
        user_id=user_id
    )


@cat.on_cmd(cmds=["bag", "背包"], always=True)
async def show_bag():
    '''展示背包；你还可以 bag @xx 查看其他人的背包'''
    if not cat.current.arg:
        money = get_money(cat.current.session_id, cat.current.sender_id)
        name = cat.current.sender_name
    else:
        user = await cat.get_user(str(cat.current.nums[0]))
        if not user:
            await cat.send("查无此人")
            return
        money = get_money(cat.current.session_id, user.id)
        name = user.name

    await cat.send(f"[{name}]当前有 {money.value}金")
